/*
	utility program for the ProWesS loader

	REXT : load a resident extension
	this program accepts one or two parameters : the name of the file which
	has to be loaded, and (optionally) a thing which should not exist. If that
	thing does exists, the resident extension should not be loaded.
	Apart from that, a searchpath may be passed (-path path), else the program
	default directory (cfr. DEVProgramGet) will be searched.

	written by Joachim Van der Auwera for PROGS
	(c) 1996 by PROGS, Professional & Graphical Software
	started 08 January 1996
*/

#include "str_h"
#include "mem_h"
#include "io_h"
#include "thing_h"

#define catch(x) if (err=(x)) return err; else

Error init(char *param)
{
	Error err;
	char **params;
	char *name=NULL, *thing=NULL, *path="$P";
	Thing dummy;
	int i;
	Channel file;
	char *base;
	Size size;

	/* pars the command line parameters */
	catch( STRBreak(param,&params) );
	for (i=0 ; params[i] ; i++)
	{
		if (STRSameCI("-path",params[i]))
		{
			if (params[i+1])
				path=params[++i];
			else
				return ERR_IPAR;
		}
		else
		{
			if (!name)
				name=params[i];
			else
				thing=params[i];
		}
	}
	if (!name) return ERR_IPAR;

	/* check if the thing exists */
	if (thing)
	{
		err=THINGUse(thing,0,&dummy,NULL);
		if (!err)
		{
			THINGFree(thing);
			return ERR_OK;
		}
	}

	/* load the resident extension */
	catch( IOOpenPath(name,OPEN_READ,path,&file) );
	catch( IOLength(file, &size) );
	catch( MEMAllocateFor(size, SYSTEM, &base) );
	err=IOFileLoad(file,size,base);
	if (err) { MEMRelease(base); return err; }
	IOClose(file);

	/* start resident extension */
	return ((Error (*)())base)();
}
